__author__ = 'alvinhkh'

import ftplib
import sys
import os
import time
from FileTransferHelper import *
from SavedFile import *
import shelve

class DirEntry:
    def __init__(self, filename, ftps, root_directory=None):
        self.filename = filename
        if root_directory == None:
            root_directory = ftps.pwd()
        try:
            ftps.cwd(filename)
            self.file_type = 'd'
            ftps.cwd(root_directory)
        except ftplib.error_perm:
            self.file_type = '-'

    def gettype(self):
        return self.file_type

    def getfilename(self):
        return self.filename


class FTPUtils:
    
    def __init__(self):
        self.ftps = ftplib.FTP_TLS(timeout=60)
        self.clock_offset = 0

        self.ftps_host = ''
        self.ftps_port = 21
        self.ftps_username = ''
        self.ftps_password = ''
        self.ftps_secure_data = False
        self.get_preference()

    def get_preference(self):
        try:
            pref_db = shelve.open('ftpsync_preference.db')
            preferences = {}
            if 'preferences' in pref_db:
                preferences = dict(pref_db['preferences'])
            if 'ftps_host' in preferences:
                self.ftps_host = preferences['ftps_host']
            if 'ftps_port' in preferences and int(preferences['ftps_port']) > 0:
                self.ftps_port = int(preferences['ftps_port'])
            if 'ftps_username' in preferences:
                self.ftps_username = preferences['ftps_username']
            if 'ftps_password' in preferences:
                self.ftps_password = preferences['ftps_password']
            if 'ftps_secure_data' in preferences:
                self.ftps_secure_data = preferences['ftps_secure_data']
            pref_db.close()
        except IOError:
            print 'Fail to open shelve file'

    def noop(self):
        try:
            print self.ftps.sendcmd('NOOP')
        except ftplib.all_errors, e:
            return False
        return True

    def get_list_directory(self, root_path):
        files = []
        try:
            n_items = self.ftps.nlst(str(root_path))
            files = [DirEntry(item, self.ftps, self.ftps.pwd()) for item in n_items]
        except ftplib.all_errors, resp:
            if str(resp) == "550 No files found":
                print "No files in this directory"
            else:
                raise
        return files
    
    def change_directory(self, path):
        try:
            self.ftps.cwd(path)
        except ftplib.error_perm, resp:
            print 'ERROR %s: cannot CD to "%s"' % (str(resp) , path)
            print "CWD:", self.ftps.pwd()
            return False
        print 'Changed to "%s" folder' % path
        
    def abspath(self, *paths):
        filename = os.path.join(*(paths or ('',)))    
        return filename
        
    def get_mtime(self, filename, directory=""):
        """
        Return modification time of the file in ftps server.
        If the file does not exist, return None.
        If the name refers to a directory, return 0.
        """
        file_path = self.abspath(directory, filename)
        try:
            resp = self.ftps.sendcmd('MDTM %s' % file_path)
        except ftplib.error_perm, resp:
            s = str(resp)
            if s.startswith('550 I can only retrieve regular files'):
                # filename is directory
                return 0
            if s.startswith('550') and "not a plain file" in s:
                # filename is directory
                return 0
            if s.startswith('550 Can\'t check for file existence'):
                # file with filename does not exist
                return None
            raise
        assert resp[:3]=='213', `resp, file_path`
        mtime = int(time.mktime(time.strptime(resp[3:].strip(), '%Y%m%d%H%M%S')))
        return mtime
        
    def set_local_mtime(self, remote_path, local_path):
        remote_mtime = self.get_mtime(remote_path)
        local_mtime = int(os.path.getmtime(local_path))
        mtime = remote_mtime - self.clock_offset
        os.utime(local_path, (mtime, mtime))
        print('modify local mtime: %s secs\n' % (local_mtime - mtime))
        
    def get_remote_files(self, directory=''):
        """
        Return a {files: {path, full_path, modification times, file size}} map of the ftp directory.

        If listing is True then .listing files are used to accelerate
        getting the map information. When .listing file does not exist,
        it will be created.

        When uploading a file or a directory then the uploader is
        responsible for removing .listing file from the parent
        directory of the uploaded file or directory. The .listing file
        should also be removed when removing a file, this is not
        necessary when removing a directory, though.  The above will
        ensure that .listing will be kept up-to-date.

        The paths to files correspond to absolute paths in the ftp
        server.
        """
        r = {}
        dirs = []
        wd = self.abspath(directory)
        r[wd] = {'path': wd, 'mtime': 0}  # directories have 0 mtime.
        print('listing directory %r ' % wd),
        lst = filter(lambda n: n not in ['.', '..'], self.ftps.nlst(wd))
        print('[%s items]: ' % (len(lst))),
        for n in lst:
            path = os.path.join(wd, n)
            mtime = self.get_mtime(path)
            if mtime == 0:
                dirs.append(path)
                print('d'),
            else:
                self.ftps.sendcmd("TYPE i")
                size = self.ftps.size(path)
                r[path] = {'path': path, 'remotepath': path, 'mtime': mtime, 'size': size}
                print('.'),
        print('')
        for path in dirs:
            r.update(**self.get_remote_files(directory=path))
        return r
    
    def get_file_list(self, remote_path):
        """
        Return a {files: {path, full_path, modification times, file size}} map of the ftp
        directory where files are relative to modification times are relative to local clock.
        """
        if remote_path == '/':
            n = 0
        else:
            assert not remote_path.endswith('/'), `remote_path`
            n = len(remote_path)
        files = {}
        for return_path, item in self.get_remote_files(directory=remote_path).items():
            if item['mtime']==0:
                # skip directories
                continue
            if os.path.basename(return_path).startswith('.'):
                continue
            assert return_path[n:n+1] in ['/',''],``return_path,n``
            fn = return_path[n+1:]
            files[fn] = {
                'path': fn, 
                'remotepath': item['remotepath'],
                'mtime': item['mtime'] - self.clock_offset, 
                'size': item['size']
            }
        return files
        
    def download(self, t_file):
        t_file.set_action("download")
        status = False
        local_path = t_file.get_local_path()
        remote_path = t_file.get_remote_path()

        try:
            self.ftps.voidcmd("TYPE I")
            data_sock, estimate_size = self.ftps.ntransfercmd("RETR %s" % remote_path)
        except ftplib.error_perm, resp:
            status = False
            if "550 No files found" in str(resp):
                t_file.set_file_not_found(True)
                print('No files in this directory')
            else:
                raise
        
        directory_name = os.path.dirname(local_path)
        if not os.path.exists(directory_name):
            os.makedirs(directory_name)
        transbytes = 0
        with open(local_path, 'wb') as fd:
            while 1:
                buf = data_sock.recv(2048)
                if not len(buf):
                    break
                fd.write(buf)
                transbytes += len(buf)
                print("Received %d " % transbytes),
                if estimate_size:
                    print("of %d bytes (%.1f%%)\r" %
                          (estimate_size, 100.0 * float(transbytes) / float(estimate_size))),
                else:
                    print("bytes\r")
            fd.close()
            status = True
        data_sock.close()
        self.ftps.voidresp()
        self.set_local_mtime(remote_path, local_path)
        
        t_file.set_status(status)
        return t_file

    def upload(self, t_file):
        verbose = True
        
        class FtpUploadTracker:
            chunk_size = 1024
            size_written = 0
            total_size = 0
            last_shown_percent = 0
            
            def __init__(self, total_size, chunk_size):
                self.total_size = total_size
                self.chunk_size = chunk_size

            def handle(self, block):
                if self.total_size > self.chunk_size:
                    self.size_written += self.chunk_size
                else:
                    self.size_written = self.total_size
                percent_complete = round((self.size_written / self.total_size) * 100)

                if self.last_shown_percent != percent_complete:
                    self.last_shown_percent = percent_complete
                    if verbose:
                        print(str(percent_complete) + " percent upload complete.")

        t_file.set_action("upload")
        status = False
        local_path = t_file.get_local_path()
        remote_path = t_file.get_remote_path()

        try:
            with open(local_path, 'rb') as fd:
                total_size = os.path.getsize(local_path)
                chunk_size = 1024
                upload_tracker = FtpUploadTracker(int(total_size), chunk_size)
                self.ftps.storbinary("STOR %s" % remote_path, fd, chunk_size, upload_tracker.handle)
                fd.close()
                status = True
        except ftplib.error_perm, resp:
            status = False
            if str(resp) == "550 No files found":
                t_file.set_file_not_found(True)
                if verbose:
                    print("No files in this directory")
            else:
                raise
        self.set_local_mtime(remote_path, local_path)
        t_file.set_status(status)
        return t_file
    """
    def upload(self, tFiles):
        returnList = []
        files_status = []
        files_file_not_found = []
        for tFile in tFiles:
            if os.path.isdir(tFile.get_local_path) == True:
                print ""
            else:
                returnList.append(self.uploadSingle(tFile))
                files_status.append(tFile.getStatus());
                files_file_not_found.append(tFile.getFileNotFound());
        return returnList
    """
    def connect(self):
        self.ftps = ftplib.FTP_TLS(timeout=30)
        try:
            self.ftps.connect(self.ftps_host, int(self.ftps_port))
            self.ftps.login(self.ftps_username, self.ftps_password)
            # switch to secure data connection..
            # IMPORTANT! Otherwise, only the user and password is encrypted and not all the file data.
            if self.ftps_secure_data:
                self.ftps.prot_p()
            self.ftps.nlst()
        except ftplib.all_errors, e:
            return {'error': True, 'message': e}
        return {'error': not self.noop()}
        
    def clock_sync(self, remote_path=""):
        """
        Check and compare local time and server time
        by creating a new file and check its last modification time.
        """
        try:
            filename = '.clocksync'
            remote_file = self.abspath(remote_path, filename)
            if not os.path.isfile(filename):
                f = open(filename,'w')
                f.write('temporary file created by ftpsync')
                f.close()
            f = open(filename, 'rb')
            self.ftps.storbinary('STOR ' + remote_file, f, 1024)
            f.close()
            local_time1 = time.time()
            remote_time = self.get_mtime(remote_file)
            local_time2 = time.time()
            self.ftps.delete(remote_file)
            os.remove(filename)
            if local_time2 < remote_time:  # remote is in future
                sync_off = int(remote_time - local_time2)
            elif local_time1 > remote_time:  # remote is in past or equal
                sync_off = int(remote_time - local_time1)
            else:
                sync_off = 0
            print('clock offset: %s\n' % sync_off)
            return sync_off
        except ftplib.error_perm, e:
            print e
            return 0
        
    def make_directory(self, path):
        verbose = True
        full_path = self.abspath(path)
        parent = os.path.dirname(full_path)
        name = os.path.basename(full_path)
        if parent != '/':
            self.make_directory(parent)
        lst = self.ftps.nlst(parent)
        if name and name not in lst:
            if verbose:
                print('creating directory %r>' % full_path)
            self.ftps.mkd(full_path)

    def remove(self, t_file):
        t_file.set_action("remove_remote")
        remote_path = t_file.get_remote_path()
        print('removing %r..>' % remote_path)
        try:
            self.ftps.delete(str(remote_path))
            t_file.set_status(True)
        except ftplib.error_perm, msg:
            if 'No such file or directory' in str(msg):
                print('removed before>')
                t_file.set_status(True)
            else:
                print('FAILED: %s>' % msg)
                t_file.set_status(False)
        return t_file

    def remove_directory(self, t_file):
        t_file.set_action("remove_remote")
        remote_path = t_file.get_remote_path()
        print('removing %r..>' % remote_path),
        lst = self.ftps.nlst(remote_path)
        for n in lst:
            if n in ['.', '..']:
                continue
            fn = os.path.join(remote_path, n)
            mtime = self.get_mtime(fn)
            if mtime == 0:
                self.remove_directory(fn)
            elif mtime:
                print('removing %r>' % fn),
                self.ftps.delete(fn)
        try:
            self.ftps.rmd(remote_path)
            t_file.set_status(True)
        except ftplib.error_perm, msg:
            if 'No such file or directory' in str(msg):
                print('removed before>'),
                t_file.set_status(True)
            else:
                print('FAILED: %s>' % msg),
                t_file.set_status(False)
        print('')
        return t_file
        
    def welcome(self):
        print('Welcome: %r' % self.ftps.getwelcome())
        print('CWD: %r' % self.ftps.pwd())
        ret = []
        self.ftps.dir("",ret.append)
        ret = [x.split()[-1] for x in ret if x.startswith("d")]
        print ret
        
    def disconnect(self):
        self.ftps.close()

    def write_save(self, array_json, local_file_path, remote_file_path):
        local_file = os.path.join(local_file_path, '.ftpsync-remote')
        write_array = []
        for item in array_json:
            write_array.append({item: array_json[item]})
        if os.path.isdir(local_file_path):
            with open(local_file, 'w') as f:
                json.dump({'root': remote_file_path, 'list': write_array}, f)
                f.close()

        remote_file = os.path.join(remote_file_path, '.ftpsync')
        try:
            with open(local_file, 'rb') as fd:
                self.ftps.storbinary("STOR %s" % remote_file, fd, 1024)
                fd.close()
        except ftplib.error_perm, resp:
            if str(resp) == "550 No files found":
                print('No files in this directory %s>' % remote_file_path)
            else:
                raise

    def get_save(self, local_file_path, remote_file_path, force_remote=False):
        self.clock_offset = 0  # self.clock_sync(remote_file_path)
        local_file = os.path.join(local_file_path, ".ftpsync-remote")
        remote_file = os.path.join(remote_file_path, ".ftpsync")
        # if not os.path.isdir(local_path) or not os.path.isfile(local_file):
        if force_remote or not os.path.isfile(local_file) or os.path.getsize(local_file) < 1:
            if '.ftpsync' in self.ftps.nlst(remote_file_path):
                try:
                    self.ftps.voidcmd("TYPE I")
                    data_sock, estimate_size = self.ftps.ntransfercmd("RETR %s" % remote_file)
                except ftplib.error_perm, resp:
                    if "550 No files found" in str(resp):
                        print('No files in directory %s>' % remote_file_path)
                    else:
                        raise

                with open(local_file, 'wb') as fd:
                    while 1:
                        buf = data_sock.recv(2048)
                        if not len(buf):
                            break
                        fd.write(buf)
                    fd.close()
                data_sock.close()
                self.ftps.voidresp()

        array_json = []
        if os.path.isfile(local_file) and os.path.getsize(local_file) > 0:
            with open(local_file, 'rb') as f:
                array_json = json.load(f)
                f.close()

        return array_json
