__author__ = 'alvinhkh'

import tkMessageBox
from Tkinter import *
from threading import Thread
import shelve


class PreferencesFrame(Frame):
    # GUI Setup
    def __init__(self, master):
        Frame.__init__(self, master)
        self.grid()
        self.master = master
        pref_db = shelve.open('ftpsync_preference.db')
        preferences = {}
        if 'preferences' in pref_db:
            preferences = dict(pref_db['preferences'])

        self.ftps_host = StringVar()
        self.ftps_port = IntVar()
        self.ftps_username = StringVar()
        self.ftps_password = StringVar()
        self.ftps_secure_data = BooleanVar()
        self.ftps_status = StringVar()
        self.ftps_status.set(' ')
        self.sync_schedule = BooleanVar()
        self.sync_interval = IntVar()
        self.sync_interval.set(5)
        self.conflict_solution = IntVar()
        self.conflict_solution.set(1)

        if 'ftps_host' in preferences:
            self.ftps_host.set(preferences['ftps_host'])
        if 'ftps_port' in preferences and int(preferences['ftps_port']) > 0:
            self.ftps_port.set(int(preferences['ftps_port']))
        if 'ftps_username' in preferences:
            self.ftps_username.set(preferences['ftps_username'])
        if 'ftps_password' in preferences:
            self.ftps_password.set(preferences['ftps_password'])
        if 'ftps_secure_data' in preferences:
            self.ftps_secure_data.set(preferences['ftps_secure_data'])
        if 'sync_schedule' in preferences:
            self.sync_schedule.set(preferences['sync_schedule'])
        if 'sync_interval' in preferences and int(preferences['sync_interval']) > 0:
            self.sync_interval.set(int(preferences['sync_interval']))
        if 'conflict_solution' in preferences and int(preferences['conflict_solution']) >= 0:
            self.conflict_solution.set(int(preferences['conflict_solution']))

        pref_db.close()
        t = Thread(target=self.check_ftps)
        t.start()

        integer_validate = (self.register(self.integer_validate), '%d', '%i', '%P', '%s', '%S', '%v', '%V', '%W')

        ftps_label = Label(self, text='FTPS Settings', font=('MS', 12, 'bold'))
        ftps_label.grid(row=0, column=0, columnspan=3, sticky=W)

        ftps_host_label = Label(self, text='Host: ', font=('MS', 10))
        ftps_host_label.grid(row=1, column=0, sticky=E)
        ftps_host_entry = Entry(self, textvariable=self.ftps_host, font=('MS', 10))
        ftps_host_entry.grid(row=1, column=1, columnspan=2, sticky=W)

        ftps_port_label = Label(self, text='Port: ', font=('MS', 10))
        ftps_port_label.grid(row=2, column=0, sticky=E)
        ftps_port_entry = Entry(self, textvariable=self.ftps_port, font=('MS', 10),
                                validate='key', validatecommand=integer_validate)
        ftps_port_entry.grid(row=2, column=1, columnspan=2, sticky=W)

        ftps_username_label = Label(self, text='Username: ', font=('MS', 10))
        ftps_username_label.grid(row=3, column=0, sticky=E)
        ftps_username_entry = Entry(self, textvariable=self.ftps_username, font=('MS', 10))
        ftps_username_entry.grid(row=3, column=1, columnspan=2, sticky=W)

        ftps_password_label = Label(self, text='Password: ', font=('MS', 10))
        ftps_password_label.grid(row=4, column=0, sticky=E)
        ftps_password_entry = Entry(self, textvariable=self.ftps_password, show="*", font=('MS', 10))
        ftps_password_entry.grid(row=4, column=1, columnspan=2, sticky=W)

        ftps_secure_data_button = Checkbutton(self, text="Secure data connection",
                                              variable=self.ftps_secure_data, onvalue=True, offvalue=False)
        ftps_secure_data_button.grid(row=5, column=0, columnspan=3, sticky=W)

        ftps_status_value = Label(self, textvariable=self.ftps_status, font=('MS', 10))
        ftps_status_value.grid(row=6, column=0, columnspan=3, sticky=E)

        sync_label = Label(self, text='Sync', font=('MS', 12, 'bold'))
        sync_label.grid(row=10, column=0, columnspan=3, sticky=W)

        sync_schedule_button = Checkbutton(self, text="Schedule Sync",
                                           variable=self.sync_schedule, onvalue=True, offvalue=False)
        sync_schedule_button.grid(row=11, column=0, columnspan=3, sticky=W)

        sync_interval_label = Label(self, text='Sync Interval', font=('MS', 10))
        sync_interval_label.grid(row=12, column=0, sticky=E)
        sync_interval_entry = Entry(self, textvariable=self.sync_interval, font=('MS', 10),
                                    validate='key', validatecommand=integer_validate)
        sync_interval_entry.grid(row=12, column=1, sticky=E+W)
        minutes_label = Label(self, text='minutes', font=('MS', 10))
        minutes_label.grid(row=12, column=2, sticky=E)

        conflict_label = Label(self, text='Sync Conflicts', font=('MS', 10))
        conflict_label.grid(row=20, column=0, columnspan=3, sticky=W)

        conflict_skip = Radiobutton(self, text="Skip", font=('MS', 10),
                                    variable=self.conflict_solution, value=0)
        conflict_skip.grid(row=21, column=0, columnspan=3, sticky=W)
        conflict_local = Radiobutton(self, text="Local First (Upload)", font=('MS', 10),
                                     variable=self.conflict_solution, value=1)
        conflict_local.grid(row=22, column=0, columnspan=3, sticky=W)
        conflict_remote = Radiobutton(self, text="Server First (Download)", font=('MS', 10),
                                      variable=self.conflict_solution, value=2)
        conflict_remote.grid(row=23, column=0, columnspan=3, sticky=W)
        conflict_user = Radiobutton(self, text="User Select", font=('MS', 10),
                                    variable=self.conflict_solution, value=3)
        # conflict_user.grid(row=24, column=0, columnspan=3, sticky=W)

        space = Label(self, text=' ')
        space.grid(row=90, column=0, columnspan=3, sticky=N+S+E)

        button_frame = Frame(self)
        button_frame.grid_columnconfigure(4, weight=1)
        button_frame.grid(row=100, column=0, columnspan=3, sticky=N+S+W+E)
        close_button = Button(button_frame, text='Close', font=('MS', 10, 'bold'), command=self.action_close)
        close_button.grid(row=0, column=0, sticky=W)
        save_button = Button(button_frame, text='Save', font=('MS', 10, 'bold'), command=self.action_save)
        save_button.grid(row=0, column=4, sticky=E)

    def check_ftps(self):
        self.ftps_status.set('Checking...')
        from FTPUtils import FTPUtils
        ftps = FTPUtils()
        connect = ftps.connect()
        if 'error' in connect and connect['error']:
            self.ftps_status.set('Fail to connect server')
            if 'message' in connect:
                self.ftps_status.set(str(connect['message']))
        else:
            self.ftps_status.set('Able to connect server')

    def action_close(self):
        print "cancel"
        self.master.destroy()

    def action_save(self):
        pref_db = shelve.open('ftpsync_preference.db')
        preferences = {}
        if 'preferences' in pref_db:
            preferences = dict(pref_db['preferences'])
        valid = False
        try:
            if self.ftps_host.get() == "" or not self.ftps_port.get() or \
               self.ftps_username.get() == "" or self.ftps_password.get() == "" or \
               not self.sync_interval.get() or not self.conflict_solution.get():
                valid = False
                tkMessageBox.showinfo("Preferences", "All field must be filled in")
            elif int(self.ftps_port.get()) < 1:
                valid = False
            elif int(self.sync_interval.get()) < 2:
                tkMessageBox.showinfo("Preferences", "Minimum Sync Interval is 2 minutes")
                valid = False
            elif int(self.sync_interval.get()) > 60:
                tkMessageBox.showinfo("Preferences", "Maximum Sync Interval is 60 minutes")
                valid = False
            else:
                valid = True
        except ValueError, e:
            valid = False
            tkMessageBox.showinfo("Preferences", "All field must be filled in")
        if valid:
            preferences['ftps_host'] = self.ftps_host.get()
            preferences['ftps_port'] = self.ftps_port.get()
            preferences['ftps_username'] = self.ftps_username.get()
            preferences['ftps_password'] = self.ftps_password.get()
            preferences['ftps_secure_data'] = self.ftps_secure_data.get()
            preferences['sync_schedule'] = self.sync_schedule.get()
            preferences['sync_interval'] = self.sync_interval.get()
            preferences['conflict_solution'] = self.conflict_solution.get()
            pref_db['preferences'] = dict(preferences)
            tkMessageBox.showinfo("Preferences", "Saved")
            t = Thread(target=self.check_ftps)
            t.start()
        pref_db.close()

    def integer_validate(self, action, index, value_if_allowed,
                         prior_value, text, validation_type, trigger_type, widget_name):
        if int(action) == 0:
            return True
        elif text in '0123456789':
            try:
                int(value_if_allowed)
                return True
            except ValueError:
                return False
        else:
            return False


root = Tk()
root.title("Preferences")
root.resizable(False, False)
app = PreferencesFrame(master=root)
app.mainloop()