__author__ = 'alvinhkh'

import os
import json
import uuid

trash_dir = '.trash_local'
ftp_trash_dir = '.trash_remote'

# TODO: client id
client_id = '8f5be32e-e08c-11e4-a0b8-80e65007d9ca' # str(uuid.uuid1())
server_id = 'server'


def ignore_filename(filename):
    root, ext = os.path.splitext(filename)
    basename = os.path.basename(filename)
    if ext in ['.pyc', '.pyo', '.backup'] or ext.endswith('~') or ext.endswith('#') or basename.startswith('.'):
        return True
    return False


def fix_dirs(dirs):
    """
    if 'CVS' in dirs: dirs.remove('CVS')
    if '.svn' in dirs: dirs.remove('.svn')
    if '.bzr' in dirs: dirs.remove('.bzr')
    """
    if trash_dir in dirs: dirs.remove(trash_dir)
    if ftp_trash_dir in dirs: dirs.remove(ftp_trash_dir)


def get_local_file_list(prev_root):
    r = {}
    wd = os.path.abspath(os.path.normpath(prev_root))
    n = len(wd)
    if n == 1:
        n = 0
    for root, dirs, files in os.walk(wd):
        fix_dirs(dirs)
        for f in files:
            if ignore_filename(f):
                continue
            fn = os.path.join(root, f)
            mtime = int(os.path.getmtime(fn))
            assert fn[n:n+1] in ['/','','\\'],`fn, n`
            path = fn[n+1:]
            size = os.path.getsize(fn)
            r[path] = {'path': path, 'localpath': fn, 'mtime': mtime, 'size': size}
    return r


def write_save(array_json, file_path):
    filename = os.path.join(file_path, ".ftpsync")
    write_array = []
    for item in array_json:
        write_array.append({item: array_json[item]})
    if os.path.isdir(file_path):
        with open(filename, 'w') as f:
            json.dump({'root': file_path, 'list': write_array}, f)
            f.close()


def get_save(file_path):
    json_file = {}
    filename = os.path.join(file_path, ".ftpsync")
    if os.path.isdir(file_path) and os.path.isfile(filename) and os.path.getsize(filename) > 0:
        with open(filename, 'rb') as f:
            json_file = json.load(f)
            f.close()
    return json_file


def read_save(file_path, json_file, replica_id):
    """
    Read save list, check version object exist or not
    """
    if 'root' in json_file and json_file['root'] == file_path and 'list' in json_file:
        array_list = json_file['list']
        json_list = {}
        for item in array_list:
            json_list.update(item)
        for item in json_list:
            if 'version' not in json_list[item]:
                json_list[item].update({'version': {replica_id: 0}})
            else:
                version_object = json_list[item]['version']
                if replica_id not in version_object:
                    version_object.update({replica_id: 0})
                json_list[item].update({'version': version_object})
        return json_list
    else:
        return False


def list_edit(file_path, add_object, old_list, replica_id):
    """
    Add or Edit save list
    """
    if not file_path:
        return old_list
    new_list = dict(old_list)
    new_object = dict(add_object)
    if 'version' not in new_object:
        new_object.update({'version': {replica_id: 0}})
    else:
        version_object = new_object['version']
        if replica_id not in version_object:
            version_object.update({replica_id: 0})
            new_object.update({'version': version_object})
    new_list.update({file_path: new_object})
    return json.loads(json.dumps(new_list))


def list_mark_as_delete(file_path, mark_object, old_list, replica_id):
    """
    Mark file as delete, save list
    """
    if not file_path:
        return old_list
    new_list = dict(old_list)
    new_object = dict(mark_object)
    file_version = increment_version(new_object, replica_id)
    new_object.update({'version': file_version})
    new_object.update({'mtime': -1})
    new_object.update({'size': -1})
    new_list.update({file_path: new_object})
    return json.loads(json.dumps(new_list))


def increment_version(file_object, replica_id):
    if not file_object:
        return file_object
    new_object = dict(file_object)
    file_version = dict({})
    if 'version' in new_object:
        file_version = new_object['version']
    if replica_id in file_version:
        file_version.update({replica_id: int(file_version[replica_id])+1})
    else:
        file_version.update({replica_id: 0})
    return file_version


def get_list_file_version(file_path, file_list, replica_id):
    if not file_path or not file_list:
        return 0
    file_list = dict(file_list)
    if file_path in file_list:
        new_object = dict(file_list[file_path])
        if 'version' in new_object:
            version_object = new_object['version']
            if replica_id in version_object:
                client_version = int(version_object[replica_id])
                return client_version
    return 0


def list_compare(saved_list, current_list, replica_id):
    return_list = dict(saved_list)
    prev_checks = {}
    current_checks = {}
    
    for saved in saved_list:
        if saved in current_list:
            prev_object = dict(saved_list[saved])
            current_object = dict(current_list[saved])

            prev_path = prev_object['path']
            prev_full_path = ""
            prev_full_path_is = ""
            if 'localpath' in prev_object:
                prev_full_path = prev_object['localpath']
                prev_full_path_is = 'localpath'
            elif 'remotepath' in prev_object:
                prev_full_path = prev_object['remotepath']
                prev_full_path_is = 'remotepath'
            prev_time = int(prev_object['mtime'])
            prev_size = int(prev_object['size'])

            current_path = current_object['path']
            current_full_path = ""
            current_full_path_is = ""
            if 'localpath' in current_object:
                current_full_path = current_object['localpath']
                current_full_path_is = 'localpath'
            elif 'remotepath' in current_object:
                current_full_path = current_object['remotepath']
                current_full_path_is = 'remotepath'
            current_time = int(current_object['mtime'])
            current_size = int(current_object['size'])

            if 'version' in prev_object:
                current_object.update({'version': prev_object['version']})
            else:
                current_object.update({'version': {}})

            if prev_time > current_time + 60 or prev_time < current_time - 60 or prev_size != current_size:
                print "mismatch: last modification time or file size"
                # Edit and Version +1
                file_version = increment_version(current_object, replica_id)
                current_object.update({'version': file_version})

                return_list = dict(list_edit(current_path, current_object, return_list, replica_id))

            prev_checks.update({saved: prev_object})
        else:
            prev_object = dict(saved_list[saved])
            # previous have, current nope, deleted
            prev_path = prev_object['path']
            prev_time = int(prev_object['mtime'])
            prev_size = int(prev_object['size'])
            if prev_time < 0 and prev_size < 0:
                print(">>> marked as deleted: %s> " % prev_object)
            else:
                print(">>> deleted after last sync: %s> " % prev_object)
                return_list = dict(list_mark_as_delete(prev_path, prev_object, return_list, replica_id))

            prev_checks.update({saved: prev_object})
    for current in current_list:
        if current not in saved_list:
            current_object = dict(current_list[current])
            # remaining, current have, previous nope, newly added
            current_path = current_object['path']

            print(">>> newly added after last sync: %s> " % current_object)
            return_list = dict(list_edit(current_path, current_object, return_list, replica_id))

            current_checks.update({current: current_object})

    return return_list