__author__ = 'alvinhkh'

from FileTransferHelper import *
from FTPUtils import *
from SavedFile import *
from VectorClock import *
from threading import Thread
import datetime
import shelve


class Sync:
    def __init__(self):
        self.ftps = FTPUtils()
        connect = self.ftps.connect()
        if 'error' in connect and not connect['error']:
            # self.ftps.change_directory('/')
            # self.ftps.welcome()
            print "Connect Success"
        else:
            error_message = 'Fail to connect server'
            if 'message' in connect:
                error_message = str(connect['message'])
            print error_message
            from sys import exit
            exit(0)

        db = shelve.open('ftpsync_preference.db')
        preferences = {}
        if 'preferences' in db:
            preferences = dict(db['preferences'])
        if 'conflict_solution' in preferences:
            self.conflict_solution = int(preferences['conflict_solution'])
        else:
            # 0: cancel, 1: upload(local first), 2: download(remote first), 3: user select (todo)
            self.conflict_solution = 0
        print "conflict_solution: ", self.conflict_solution
        db.close()

        # clear prev conflicts
        log_db = shelve.open('ftpsync_log.db')
        log_db['conflicts'] = []
        log_db.close()

    def timestamp_to_date(self, timestamp):
        return datetime.datetime.fromtimestamp(
            int(timestamp)
        ).strftime('%Y-%m-%d %H:%M:%S')

    def action_download(self, file_helper):
        file_helper = self.ftps.download(file_helper)
        self.action_add_log({
            'local': file_helper.get_local_path(),
            'remote': file_helper.get_remote_path(),
            'action': file_helper.get_action(),
            'status': file_helper.get_status(),
            'skip': file_helper.get_skip(),
            'date': time.time(),
            'file': file_helper
        })
        return file_helper

    def action_upload(self, file_helper):
        file_helper = self.ftps.upload(file_helper)
        self.action_add_log({
            'local': file_helper.get_local_path(),
            'remote': file_helper.get_remote_path(),
            'action': file_helper.get_action(),
            'status': file_helper.get_status(),
            'skip': file_helper.get_skip(),
            'date': time.time(),
            'file': file_helper
        })
        return file_helper

    def action_remove_remote(self, file_helper):
        file_helper = self.ftps.remove(file_helper)
        self.action_add_log({
            'local': file_helper.get_local_path(),
            'remote': file_helper.get_remote_path(),
            'action': file_helper.get_action(),
            'status': file_helper.get_status(),
            'skip': file_helper.get_skip(),
            'date': time.time(),
            'file': file_helper
        })
        return file_helper

    def action_delete_local(self, file_helper):
        file_helper.set_action("delete_local")
        local_file_path = file_helper.get_local_path()
        print('deleting %r..>' % local_file_path)
        if os.path.isfile(local_file_path):
            os.remove(local_file_path)
            file_helper.set_status(True)
        else:
            if os.path.exists(local_file_path):
                print("Error: %s file not found>" % local_file_path)
                file_helper.set_status(False)
            else:
                print('deleted before>')
                file_helper.set_status(True)
        if not (file_helper.get_local_time() < 0 and file_helper.get_local_size()):
            self.action_add_log({
                'local': file_helper.get_local_path(),
                'remote': file_helper.get_remote_path(),
                'action': file_helper.get_action(),
                'status': file_helper.get_status(),
                'skip': file_helper.get_skip(),
                'date': time.time(),
                'file': file_helper
            })
        return file_helper

    def action_conflict(self, file_helper):
        file_helper.set_action("conflict")
        print "conflict please choose"
        print file_helper.get_local_path(), self.timestamp_to_date(file_helper.get_local_time()), str(file_helper.get_local_time()), str(file_helper.get_local_size())
        print file_helper.get_remote_path(), self.timestamp_to_date(file_helper.get_remote_time()), str(file_helper.get_remote_time()), str(file_helper.get_remote_size())
        self.action_add_log({
            'local': file_helper.get_local_path(),
            'remote': file_helper.get_remote_path(),
            'action': file_helper.get_action(),
            'status': file_helper.get_status(),
            'skip': file_helper.get_skip(),
            'date': time.time(),
            'file': file_helper
        })
        if (self.conflict_solution == 2):
            self.action_download(file_helper)
        elif (self.conflict_solution == 1):
            self.action_upload(file_helper)
        else:
            """
            log_db = shelve.open('ftpsync_log.db')
            conflict_files = []
            if 'conflicts' in log_db:
                conflict_files = log_db['conflicts']
            conflict_files.append(file_helper)
            log_db['conflicts'] = conflict_files
            log_db.close()
            """
            file_helper.set_skip(True)
        return file_helper

    def action_skip(self, file_helper):
        file_helper.set_action("skip")
        file_helper.set_status(True)
        file_helper.set_skip(True)
        print "Skip ", file_helper.get_local_path(), file_helper.get_remote_path()
        self.action_add_log({
            'local': file_helper.get_local_path(),
            'remote': file_helper.get_remote_path(),
            'action': file_helper.get_action(),
            'status': file_helper.get_status(),
            'skip': file_helper.get_skip(),
            'date': time.time(),
            'file': file_helper
        })
        return file_helper

    def action_add_log(self, log):
        log_db = shelve.open('ftpsync_log.db')
        if 'sync_log' in log_db:
            sync_log = list(log_db['sync_log'])
        else:
            sync_log = []
        sync_log.append(log)
        log_db['sync_log'] = sync_log
        log_db.close()

    def action_sync(self, local_root_path, remote_root_path):
        saved_local_list = read_save(local_root_path, get_save(local_root_path), client_id)
        print "saved_local_list", saved_local_list
        latest_local_list = dict(get_local_file_list(local_root_path))
        # print "latest_local_list ", latest_local_list
        if not saved_local_list:
            saved_local_list = dict({})
        local_list = dict(list_compare(saved_local_list, latest_local_list, client_id))
        print "local_list ", local_list
        print ""

        saved_remote_list = read_save(remote_root_path, self.ftps.get_save(local_root_path, remote_root_path), server_id)
        print "saved_remote_list", saved_remote_list
        if not saved_remote_list:
            saved_remote_list = dict({})
        latest_remote_list = dict(self.ftps.get_file_list(remote_root_path))
        # print "latest_remote_list", latest_remote_list
        print ""
        remote_list = dict(list_compare(saved_remote_list, latest_remote_list, server_id))
        print "remote_list ", remote_list
        print ""

        local_checks = dict({})
        remote_checks = dict({})
        for local in local_list:
            if local in remote_list:
                local_object = dict(local_list[local])
                remote_object = dict(remote_list[local])
                tf = FileTransferHelper(local_absolute_path=local_object['localpath'], remote_absolute_path=remote_object['remotepath'])
                local_time = int(local_object['mtime'])
                tf.set_local_time(local_time)
                remote_time = int(remote_object['mtime'])
                tf.set_remote_time(remote_time)
                local_size = int(local_object['size'])
                tf.set_local_size(local_size)
                remote_size = int(remote_object['size'])
                tf.set_remote_size(remote_size)
                local_version = dict(local_object['version'])
                remote_version = dict(remote_object['version'])
                vc_max = dict_max(local_version, remote_version)
                vc_compare = dict_compare(local_version, remote_version)
                local_checks.update({local: tf})

                # print local
                # print local_object
                # print remote_object
                if vc_compare is dict_equal:
                    # print "vc compare: EQUAL , " + tf.get_local_path()
                    if local_size != remote_size or local_time != remote_time:
                        print "TRANSFER.CONFLICT"
                        self.action_conflict(tf)
                    else:
                        print "TRANSFER.SKIP"
                        self.action_skip(tf)
                elif vc_compare is dict_greater:
                    # print "vc compare: GREATER , upload / remove remote " + tf.get_remote_path()
                    if local_time < 0 and local_size < 0:
                        print "TRANSFER.REMOVE_REMOTE"
                        self.action_remove_remote(tf)
                    else:
                        print "TRANSFER.UPLOAD"
                        self.action_upload(tf)
                elif vc_compare is dict_smaller:
                    # print "vc compare: SMALLER , download / delete local " + tf.get_local_path()
                    if remote_time < 0 and remote_size < 0:
                        print "TRANSFER.DELETE_LOCAL"
                        self.action_delete_local(tf)
                    else:
                        print "TRANSFER.DOWNLOAD"
                        self.action_download(tf)
                else:
                    # print "vc compare: SIMULTANEOUS, conflict " + tf.get_local_path()
                    if local_time < 0 and local_size < 0:
                        print "TRANSFER.REMOVE_REMOTE"
                        self.action_remove_remote(tf)
                    if remote_time < 0 and remote_size < 0:
                        print "TRANSFER.DELETE_LOCAL"
                        self.action_delete_local(tf)
                    if local_time >= 0 and local_size >= 0 and remote_time >= 0 and remote_size >= 0:
                        if local_time == remote_time and local_size == remote_size:
                            print "TRANSFER.SKIP"
                            self.action_skip(tf)
                        else:
                            print "TRANSFER.CONFLICT"
                            self.action_conflict(tf)

                local_version = dict(vc_max)
                remote_version = dict(vc_max)
                local_object.update({'version': local_version})
                remote_object.update({'version': remote_version})
                local_list.update({local: local_object})
                remote_list.update({local: remote_object})
                print ""
            else:
                # local have, server nope, upload
                local_object = dict(local_list[local])
                tf = FileTransferHelper(local_absolute_path=local_object['localpath'], remote_absolute_path=remote_root_path + '/' + local)
                local_time = int(local_object['mtime'])
                tf.set_local_time(local_time)
                tf.set_remote_time(local_time)
                local_size = int(local_object['size'])
                tf.set_local_size(local_size)
                tf.set_remote_size(local_size)
                local_version = dict(local_object['version'])
                remote_version = {'server': 0}
                vc_max = dict_max(local_version, remote_version)
                local_checks.update({local: tf})

                print local, local_object
                if local_time < 0 and local_size < 0:
                    print "TRANSFER.REMOVE_REMOTE"
                    self.action_remove_remote(tf)
                else:
                    print "TRANSFER.UPLOAD"
                    self.action_upload(tf)

                local_version = dict(vc_max)
                remote_version = dict(vc_max)
                local_object.update({'version': local_version})
                remote_object = {
                    'path': local,
                    'mtime': local_time,
                    'size': local_size,
                    'remotepath': remote_root_path + '/' + local,
                    'version': remote_version
                }
                local_list.update({local: local_object})
                remote_list.update({local: remote_object})
                print ""
        for remote in remote_list:
            if remote not in local_list:
                # remaining, server have, local nope, download
                remote_object = dict(remote_list[remote])
                tf = FileTransferHelper(local_absolute_path=local_root_path + '/' + remote, remote_absolute_path=remote_object['remotepath'])
                remote_time = int(remote_object['mtime'])
                tf.set_local_time(remote_time)
                tf.set_remote_time(remote_time)
                remote_size = int(remote_object['size'])
                tf.set_local_size(remote_size)
                tf.set_remote_size(remote_size)
                local_version = {client_id : 0}
                remote_version = dict(remote_object['version'])
                vc_max = dict_max(local_version, remote_version)
                remote_checks.update({remote: tf})

                print remote, remote_object
                if remote_time < 0 and remote_size < 0:
                    print "TRANSFER.DELETE_LOCAL"
                    self.action_delete_local(tf)
                else:
                    print "TRANSFER.DOWNLOAD"
                    self.action_download(tf)

                local_version = dict(vc_max)
                remote_version = dict(vc_max)
                local_object = {
                    'path': remote,
                    'mtime': remote_time,
                    'size': remote_size,
                    'localpath': local_root_path + '/' + remote,
                    'version': local_version
                }
                remote_object.update({'version': remote_version})
                local_list.update({remote: local_object})
                remote_list.update({remote: remote_object})
                print ""

        print "local_list: ", local_list
        print "remote_list: ", remote_list
        write_save(local_list, local_root_path)
        self.ftps.write_save(remote_list, local_root_path, remote_root_path)


    def disconnect(self):
        self.ftps.disconnect()


def run_sync():
    file_sync = Sync()
    # file_sync.action_sync("./Folder", "/Public/Folder Two")
    db = shelve.open('ftpsync_dir.db')
    db_count = len(db)
    sync_path = []
    if db_count > 0:
        if 'sync_path' in db:
            sync_path = list(db['sync_path'])
    else:
        print 'Error: missing database'
    for j, path in enumerate(sync_path):
        file_sync.action_sync(path['local'], path['remote'])
    db.close()
    file_sync.disconnect()