__author__ = 'alvinhkh'

from Tkinter import *
import tkFileDialog
import tkMessageBox

from FTPUtils import *
from threading import Thread
import shelve


class UpdateSelectionFrame(Frame):
    # GUI Setup
    def __init__(self, parent, root):
        # initialise UpdateSelectionFrame Class
        Frame.__init__(self, parent)
        self.grid()
        self.grid_rowconfigure(4, weight=1)
        self.grid_columnconfigure(5, weight=1)
        self.parent = parent
        self.root = root
        # self.root.protocol("WM_DELETE_WINDOW", self.clear_selection)

        self.ftps = FTPUtils()
        self.connected = False
        # self.ftps.connect()

        self.local_path = StringVar()
        self.remote_path = StringVar()
        self.prev_local_path = ""
        self.prev_remote_path = ""
        self.remote_list_box = Listbox(self, height=4, width=40)
        self.local_path.set('/')
        self.remote_path.set('/')


    def heading(self):
        # Set Heading from root frame to display
        return "Add Sync Directory"

    def refresh(self, **kwargs):
        if 'local_path' in kwargs:
            self.prev_local_path = kwargs['local_path']
            self.local_path.set(kwargs['local_path'])
        if 'remote_path' in kwargs:
            self.prev_remote_path = kwargs['remote_path']
            self.remote_path.set(kwargs['remote_path'])

        self.populate()

    def populate(self):
        local_path_label = Label(self, text='Local: ', font=('MS', 12, 'bold'))
        local_path_label.grid(row=0, column=0, sticky=E)
        local_path_entry = Entry(self, width=38, textvariable=self.local_path)
        local_path_entry.grid(row=0, column=1, columnspan=4)

        local_button = Button(self, text='Choose', command=self.ask_local_directory)
        local_button.grid(row=0, column=5, columnspan=2, sticky=E)

        remote_path_label = Label(self, text='Remote: ', font=('MS', 12, 'bold'))
        remote_path_label.grid(row=1, column=0, sticky=E)
        remote_path_entry = Entry(self, width=38, textvariable=self.remote_path)
        remote_path_entry.grid(row=1, column=1, columnspan=4)
        remote_button = Button(self, text='Update', command=self.ask_remote_directory)
        remote_button.grid(row=1, column=5, columnspan=2, sticky=E)

        scroll = Scrollbar(self, orient='vertical')
        self.remote_list_box.configure(yscrollcommand=scroll.set)
        scroll.config(command=self.remote_list_box.yview)
        self.remote_list_box.grid_forget()
        scroll.grid(row=2, column=5, sticky=W+N+S)

        t = Thread(target=self.ask_remote_directory)
        t.start()

        self.create_buttons()

    def ask_local_directory(self):
        """Returns a selected directory name."""
        options = dict()
        options['initialdir'] = self.local_path.get()
        options['mustexist'] = False
        options['parent'] = self.root
        options['title'] = 'Choose Local Directory'
        path = tkFileDialog.askdirectory(**options)
        if path:
            self.local_path.set(path)
        return path

    def ask_remote_directory(self, value=""):
        if value == "":
            value = self.remote_path.get()
        self.remote_list_box.grid(row=2, column=1, columnspan=3, sticky=W)
        self.remote_list_box.delete(0, END)
        self.remote_list_box.insert(0, 'Loading...')
        self.remote_list_box.unbind_all('<<ListboxSelect>>')
        self.remote_list_box.unbind_all('<Double-Button-1>')
        t = Thread(target=self.update_remote_list_box, args=(value,))
        t.start()
        # self.update_remote_list_box(self.remote_path.get())

    def select_remote_list_box(self, event):
        widget = event.widget
        selection = widget.curselection()
        value = widget.get(selection[0])
        self.remote_path.set(value)

    def click_remote_list_box(self, event):
        widget = event.widget
        selection = widget.curselection()
        value = widget.get(selection[0])
        print "selection:", selection, ": '%s'" % value
        self.ask_remote_directory(value)

    def update_remote_list_box(self, path, tried=False):
        if not self.connected:
            connect = self.ftps.connect()
            if 'error' in connect and not connect['error']:
                error_message = 'Fail to connect server'
                if 'message' in connect:
                    error_message = str(connect['message'])
                print error_message
                self.connected = True
        self.remote_path.set(os.path.abspath(path))
        try:
            remote_directories = self.ftps.get_list_directory(path)
        except ftplib.error_perm, e:
            self.connected = False
            if not tried:
                return self.update_remote_list_box(self, path, tried=True)
        print "%d entries:" % len(remote_directories)
        self.remote_list_box.delete(0, END)
        self.remote_list_box.insert(0, os.path.abspath(os.path.join(path, os.pardir)))
        self.remote_list_box.insert(END, os.path.abspath(path))
        if len(remote_directories) > 0:
            for i, item in enumerate(remote_directories):
                print "%s: type %s" % (item.getfilename(), item.gettype())
                if item.gettype() == 'd':
                    self.remote_list_box.insert(END, item.getfilename())
        # Update list box events
        self.remote_list_box.selection_set(1)
        self.remote_list_box.bind('<<ListboxSelect>>', self.select_remote_list_box)
        self.remote_list_box.bind('<Double-Button-1>', self.click_remote_list_box)

    def create_buttons(self):
        # create widgets to set up submit and back buttons
        btn_cancel = Button(self, text='Back', font=('MS', 12, 'bold'))
        btn_cancel['command'] = self.action_exit
        btn_cancel.grid(row=16, column=0, columnspan=2, sticky=E)

        btn_submit = Button(self, text='Submit', font=('MS', 12, 'bold'))
        btn_submit['command'] = self.action_save
        btn_submit.grid(row=16, column=2, columnspan=2, sticky=E)

    def action_exit(self):
        # Clear
        # self.remote_list_box.selection_clear(0, END)
        # self.remote_list_box.selection_set(END)
        self.local_path.set('')
        self.remote_path.set('')
        self.prev_local_path = ""
        self.prev_remote_path = ""
        self.root.show_frame("view_selection")
         
    def action_save(self):
        # Store the results of the SelectionGUI
        message = ""
        finish = False
         
        if message == "":
            db = shelve.open('ftpsync_dir.db')
            db_count = len(db)
            if 'sync_path' in db:
                sync_path = db['sync_path']
            else:
                sync_path = []
            entry = {'local': self.local_path.get(), 'remote': self.remote_path.get()}
            prev = {'local': self.prev_local_path, 'remote': self.prev_remote_path}
            if prev in sync_path:
                sync_path.remove(prev)
            if entry not in sync_path:
                sync_path.append(entry)
                tkMessageBox.showinfo("Select Directory", "Saved")
                db['sync_path'] = sync_path
                finish = True
            else:
                tkMessageBox.showinfo("Select Directory", "Existed")
            db.close()
        else:
            tkMessageBox.showwarning("Entry Error", message)

        if finish:
            self.action_exit()