__author__ = 'alvinhkh'


dict_equal = "EQUAL"
dict_greater = "GREATER"
dict_smaller = "SMALLER"
dict_simultaneous = "SIMULTANEOUS"


def dict_max(clock_one, clock_two):
    """
    VectorClock merging operation. Creates a new VectorClock with the maximum for
    each element in either clock. Used in Buffer and Process to manipulate clocks.

    :param clock_one: First Clock being merged.
    :param clock_two: Second Clock being merged.
    :return: A new VectorClock with the maximum for each element in either clock.
    """
    # Create new Clock.
    _result = {}

    # Go over all elements in clock One, put them in the new clock.
    for _entry in clock_one:
        _result.update({_entry: clock_one.get(_entry)})

    # Go over all elements in clock Two,
    for _entry in clock_two:
        # Insert the Clock Two value if it is not present in One, or if it is higher.
        if _entry not in _result or _result.get(_entry) < clock_two.get(_entry):
            _result.update({_entry: clock_two.get(_entry)})

    # Return the merged clock.
    return _result

def dict_compare(clock_one, clock_two):
    """
    VectorClock compare operation.
    Returns one of four possible values indicating how clock one relates to clock two:
    GREATER       If One > Two.
    EQUAL         If One = Two.
    SMALLER       If One < Two.
    SIMULTANEOUS  If One <> Two.

    :param clock_one: First Clock being compared.
    :param clock_two: Second Clock being compared.
    :return: VectorClock.Comparison value indicating how One relates to Two.
    """
    # Initially we assume it is all possible things.
    _equal = True
    _greater = True
    _smaller = True

    # Go over all elements in Clock one.
    for _entry in clock_one:
        # Compare if also present in clock two.
        if _entry in clock_two:
            # If there is a difference, it can never be equal.
            # Greater / smaller depends on the difference.
            if clock_one.get(_entry) < clock_two.get(_entry):
                _equal = False
                _greater = False
            if clock_one.get(_entry) > clock_two.get(_entry):
                _equal = False
                _smaller = False
        # Else assume zero (default value is 0).
        elif clock_one.get(_entry) != 0:
            _equal = False
            _smaller = False

    # Go over all elements in Clock two.
    for _entry in clock_two:
        # Only elements we have not found in One still need to be checked.
        if _entry not in clock_one and clock_two.get(_entry) != 0:
            _equal = False
            _greater = False

    # Return based on determined information.
    if _equal:
        return dict_equal
    elif _greater and not _smaller:
        return dict_greater
    elif _smaller and not _greater:
        return dict_smaller
    else:
        return dict_simultaneous

# print dict_compare({'client': 0, 'server': 1}, {'client': 1, 'server': 0})
# print dict_max({'client': 0, 'server': 1}, {'client': 1, 'server': 0})