__author__ = 'alvinhkh'

from Tkinter import *
import tkMessageBox

from FTPUtils import *
from threading import Thread
import shelve

class ViewLogFrame(Frame):
    # GUI Setup
    def __init__(self, parent, root):
        # initialise ViewLogFrame Class
        Frame.__init__(self, parent)
        self.parent = parent
        self.root = root

        self.log_db = {}
        self.sync_status = StringVar()
        self.sync_status.set('[Not Running]')

    def heading(self):
        # Set Heading from root frame to display
        return "Log"

    def refresh(self, **kwargs):
        for widget in self.winfo_children():
            widget.destroy()
        self.populate()

    def populate(self):
        self.log_db = shelve.open('ftpsync_log.db')
        self.grid()
        self.grid_rowconfigure(2, weight=1)
        self.grid_columnconfigure(1, weight=1)
        db_count = len(self.log_db)
        sync_log = []
        if db_count > 0:
            if 'sync_log' in self.log_db:
                sync_log = self.log_db['sync_log']
            else:
                print 'Error: missing database'
        self.log_db.close()

        button_frame = Frame(self)
        button_frame.grid(row=0, column=0, sticky=N+S+E+W)

        sync_button = Button(button_frame, text='Sync', font=('MS', 12, 'bold'), command=self.action_sync)
        sync_button.grid(row=0, column=0, sticky=N+S)
        sync_label = Label(button_frame, font=('MS', 12), textvariable=self.sync_status)
        sync_label.grid(row=0, column=1, sticky=N+S)
        select_button = Button(button_frame, text='Selected Directories', font=('MS', 12, 'bold'),
                               command=self.action_select_directory)
        select_button.grid(row=0, column=2, sticky=N+S)
        refresh_button = Button(button_frame, text='Refresh', font=('MS', 12, 'bold'), command=self.refresh)
        refresh_button.grid(row=0, column=3, sticky=N+S)
        clear_button = Button(button_frame, text='Clear Log', font=('MS', 12, 'bold'), command=self.action_clear)
        clear_button.grid(row=0, column=4, sticky=N+S)

        content_frame = Frame(self)
        content_frame.grid(row=1, column=0, sticky=N+S+E+W)
        i = 2
        for j, log in enumerate(reversed(sync_log)):
            action_label = Label(content_frame, text=log['action'], font=('MS', 12, 'bold'))
            action_label.grid(row=i, rowspan=3, column=0, sticky=N+S+W+E)

            status_frame = Frame(content_frame)
            status_frame.grid(row=i, rowspan=3, column=8, sticky=N+S+E+W)
            status_space = Label(status_frame, text=" ")
            status_space.grid(row=0, column=0, sticky=E+W)
            status_text = 'Done' if log['status'] else 'False'
            status_label = Label(status_frame, text=status_text, justify=CENTER, font=('MS', 12, 'bold'))
            status_label.grid(row=1, column=0, sticky=W+E)
            skip_text = 'Skipped' if log['skip'] else ''
            skip_label = Label(status_frame, text=skip_text, justify=CENTER, font=('MS', 12, 'bold'))
            skip_label.grid(row=2, column=0, sticky=W+E)

            local_path_label = Label(content_frame, text='Local: ', font=('MS', 10, 'bold'))
            local_path_label.grid(row=i, column=1, sticky=E)
            local_path_value = Label(content_frame, text=log['local'], font=('MS', 10), justify=LEFT, wraplengt=300)
            local_path_value.grid(row=i, column=2, columnspan=5, sticky=W)
            i += 1
            remote_path_label = Label(content_frame, text='Remote: ', font=('MS', 10, 'bold'))
            remote_path_label.grid(row=i, column=1, sticky=E)
            remote_path_value = Label(content_frame, text=log['remote'], font=('MS', 10), justify=LEFT, wraplengt=300)
            remote_path_value.grid(row=i, column=2, columnspan=5, sticky=W)
            i += 1
            date_value = Label(content_frame, text=self.timestamp_to_date(log['date']),
                               font=('MS', 8), justify=LEFT, wraplengt=300)
            date_value.grid(row=i, column=1, columnspan=6, sticky=E)
            i += 1
            space = Label(content_frame, text=" ")
            space.grid(row=i, column=0, columnspan=10, sticky=E+W)
            i += 1

    def action_clear(self):
        self.log_db = shelve.open('ftpsync_log.db')
        self.log_db['sync_log'] = []
        self.log_db['conflicts'] = []
        self.log_db.close()
        self.refresh()

    def action_select_directory(self):
        self.root.show_frame("view_selection")

    def action_sync(self):
        t = Thread(target=self.run_sync)
        t.start()

    def run_sync(self):
        import Sync
        self.sync_status.set('[Started]')
        Sync.run_sync()
        self.sync_status.set('[Finished]')
        self.refresh()

    def timestamp_to_date(self, timestamp):
        from datetime import datetime
        return datetime.fromtimestamp(
            int(timestamp)
        ).strftime('%Y-%m-%d %H:%M:%S')