__author__ = 'alvinhkh'

from Tkinter import *


class WindowFrame(Frame):
    def __init__(self, master=None):
        # initialise Class
        Frame.__init__(self, master)
        self.grid()
        self.master.rowconfigure(2, weight=1)
        self.master.columnconfigure(0, weight=1)
        self.master.title('Main Screen')
        
        # initialise variables
        self.child = Frame()

        header_frame = Frame(master, bg='darkblue')
        header_frame.grid(row=0, column=0, columnspan=10, sticky=W+E+N+S)
        self.heading = StringVar()
        self.heading.set('Hello')
        title_label = Label(header_frame, bg='darkblue', foreground='white',
                            textvariable=self.heading, font=('MS', 20), justify=LEFT)
        title_label.grid(row=0, column=0, ipadx=28, ipady=20, sticky=W+E+N+S)

        self.main_canvas = Canvas(master, borderwidth=0, background='#ffffff')
        self.main_frame = Frame(self.main_canvas, background='#ffffff')
        self.scrollbar = Scrollbar(master, orient='vertical', command=self.main_canvas.yview)
        self.main_canvas.configure(yscrollcommand=self.scrollbar.set)

        self.scrollbar.grid(row=1, rowspan=2, column=1, sticky=E+N+S)
        self.main_canvas.grid(row=1, rowspan=2, column=0, columnspan=9, sticky=W+E+N+S)
        self.main_canvas.grid_rowconfigure(0, weight=1)
        self.main_canvas.grid_columnconfigure(0, weight=1)
        self.main_canvas.create_window(0, 0, window=self.main_frame, anchor=N+W, tags='self.main_frame')
        self.main_frame.bind('<Configure>', self.scroll_frame)

        # Sub Frame
        self.frames = {} # Set of Frames
        # Add/Edit Selection
        from UpdateSelectionFrame import UpdateSelectionFrame
        self.frames['add_selection'] = UpdateSelectionFrame(self.main_frame, self)
        # View Selection Frame
        from ViewSelectionFrame import ViewSelectionFrame
        self.frames['view_selection'] = ViewSelectionFrame(self.main_frame, self)
        # View Log Frame
        from ViewLogFrame import ViewLogFrame
        self.frames['view_log'] = ViewLogFrame(self.main_frame, self)
        
        # Call the first Frame: View Selection
        self.show_frame('view_log')

    def scroll_frame(self, event):
        """Reset the scroll region to encompass the inner frame"""
        self.main_canvas.configure(scrollregion=self.main_canvas.bbox('all'))
        
    def show_frame(self, c, **kwargs):
        # Show a frame for the given class
        for f in self.frames:
            # hide other frames
            self.frames[f].grid_remove()
        if c in self.frames:
            frame = self.frames[c]
        else:
            return False

        # Refresh frame
        if 'refresh' in dir(frame):
            if hasattr(frame.refresh, '__call__'):
                frame.refresh(**kwargs)

        # Show frame
        frame.grid(row=0, column=0, sticky=W+E+N+S)

        # Set background
        if 'bg' in dir(frame):
            frame.config(bg=frame.bg)
        
        frame.update_idletasks()
        frame.tkraise()
        # Set title
        if 'heading' in dir(frame):
            if hasattr(frame.heading, '__call__'):
                self.heading.set(frame.heading())
        self.child = frame
        return frame
            
    def quit(self):
        # Close Window
        self.master.destroy()


root = Tk()
root.minsize(460,100)
root.resizable(False, False)
app = WindowFrame(master=root)
app.mainloop()